export declare type App = {
    name: string;
    slug?: string;
    description?: string;
    tagline?: string;
    url?: string | URL;
    logo?: string | URL | Image;
    company?: Company;
    businessModel?: LeanCanvas;
    brand?: Brand;
    marketing?: Marketing;
    category?: Category | Category[];
    alternatives?: {
        [key: string]: Alternative;
    };
    features?: Feature[];
    benefits?: Benefit[];
    nouns?: {
        [key: string]: Noun;
    };
    functions?: {
        [key: string]: Function;
    };
    triggers?: {
        [key: string]: Trigger;
    };
    searches?: {
        [key: string]: Search;
    };
    actions?: {
        [key: string]: Action;
    };
    authentication?: {
        [key: string]: AuthenticationProvider;
    };
    integrations?: Integrations;
};
export declare const generateApp: (app: App) => App;
export declare type AppContext = {
    app: App;
    env: {};
    user: User;
};
export declare type Persona = string | {
    name: string;
    description?: string;
};
export declare type Problem = string | {
    problem: string | string[];
    existingAlternatives?: string | string[];
};
export declare type RevenueStream = string;
export declare type Solution = string;
export declare type UniqueValueProp = string;
export declare type Channel = string;
export declare type KeyMetric = string;
export declare type CostStructure = string;
export declare type UnfairAdvantage = string;
export declare type LeanCanvas = {
    customerSegments?: Persona | Persona[];
    problem?: Problem | Problem[];
    revenueStreams?: RevenueStream | RevenueStream[];
    solution?: Solution | Solution[];
    uniqueValueProp?: UniqueValueProp | UniqueValueProp[];
    channels?: Channel | Channel[];
    keyMetrics?: KeyMetric | KeyMetric[];
    costStructure?: CostStructure | CostStructure[];
    unfairAdvantage?: UnfairAdvantage | UnfairAdvantage[];
};
export declare type Image = {
    src: string;
    height?: number;
    width?: number;
};
export declare type ABTester<T> = {
    value: T;
    experiment?: string;
    name?: string;
    weight?: number;
};
export declare type Function = <T = void>() => T;
export declare type Company = {
    name?: string;
    legalName?: string;
    url?: string;
    address?: string;
    address2?: string;
    city?: string;
    state?: string;
    zip?: string;
    country?: string;
    phone?: string;
    email?: string;
};
export declare type Marketing = {};
export declare type Brand = {
    style: string;
    color: string;
};
export declare type Integrations = {
    googleAnalytics?: string;
    googleTagManager?: string;
    sentry?: string;
    rollbar?: string;
    segment?: string;
    moesif?: string;
    readme?: string;
    intercom?: string;
    mixpanel?: string;
    crisp?: string;
    stripe?: {
        publishableKey: string;
        secretKey: string;
    };
};
export declare type Feature = string | {
    name: string;
    description?: string;
    icon?: string;
};
export declare type Benefit = string | {
    name: string;
    description?: string;
    icon?: string;
};
export declare type User = {};
export declare type MultiTenantCRUD = {
    tenant: Tenant;
};
export declare type Tenant = {};
export declare type Role = {};
export declare type AuthenticationProvider = {};
export declare type Nouns = {
    [key: string]: Noun;
};
export declare const OneToMany: <T extends Noun<T>>(obj?: T | undefined) => T | undefined;
export declare const OneToOne: <T extends Noun<T>>(obj?: T | undefined) => T | undefined;
export declare const ManyToOne: <T extends Noun<T>>(obj?: T | undefined) => T | undefined;
export declare const ManyToMany: <T extends Noun<T>>(obj?: T | undefined) => T | undefined;
export declare type Noun<T = void> = {
    name?: string;
    plural?: string;
    description?: string;
    schema?: Schema;
    onCreate?: (instance: T, ctx: AppContext) => void | T;
    onUpdate?: (instance: T, ctx: AppContext) => void | T;
    onDelete?: (instance: T, ctx: AppContext) => void | T;
};
export declare type Schema = {};
export declare type Trigger = {
    name?: string;
};
export declare type Search = {
    name?: string;
};
export declare type Action = {
    name?: string;
};
export declare type Alternative = {
    name?: string;
    url?: string;
    description?: string;
    pros?: string[];
    cons?: string[];
};
declare enum Category {
    Accounting = "Accounting",
    AdsConversion = "Ads & Conversion",
    Analytics = "Analytics",
    AppBuilder = "App Builder",
    AppFamilies = "App Families",
    BookmarkManagers = "Bookmark Managers",
    BusinessIntelligence = "Business Intelligence",
    Calendar = "Calendar",
    CallTracking = "Call Tracking",
    CMS = "Website Builders",
    Commerce = "Commerce",
    Communication = "Communication",
    ContactManagement = "Contact Management",
    ContentFiles = "Content & Files",
    CRM = "CRM (Customer Relationship Management)",
    CustomerAppreciation = "Customer Appreciation",
    CustomerSupport = "Customer Support",
    Dashboards = "Dashboards",
    Databases = "Databases",
    DeveloperTools = "Developer Tools",
    Devices = "Devices",
    Documents = "Documents",
    DripEmails = "Drip Emails",
    eCommerce = "eCommerce",
    Education = "Education",
    Email = "Email",
    EmailNewsletters = "Email Newsletters",
    EventManagement = "Event Management",
    Fax = "Fax",
    FileManagementStorage = "File Management & Storage",
    Filters = "Filters",
    Fitness = "Fitness",
    FormsSurveys = "Forms & Surveys",
    Fundraising = "Fundraising",
    Gaming = "Gaming",
    HRTalentRecruitment = "HR Talent & Recruitment",
    HumanResources = "Human Resources",
    ImagesDesign = "Images & Design",
    InternetOfThings = "Internet of Things",
    ProposalInvoiceManagement = "Proposal & Invoice Management",
    ITOperations = "IT Operations",
    OnlineCourses = "Online Courses",
    LifestyleEntertainment = "Lifestyle & Entertainment",
    Marketing = "Marketing",
    MarketingAutomation = "Marketing Automation",
    NewsLifestyle = "News & Lifestyle",
    Notes = "Notes",
    Notifications = "Notifications",
    PaymentProcessing = "Payment Processing",
    PhoneSMS = "Phone & SMS",
    Printing = "Printing",
    ProductManagement = "Product Management",
    Productivity = "Productivity",
    ProjectManagement = "Project Management",
    Reviews = "Reviews",
    SalesCRM = "Sales & CRM",
    SchedulingBooking = "Scheduling & Booking",
    SecurityIdentityTools = "Security & Identity Tools",
    ServerMonitoring = "Server Monitoring",
    Signatures = "Signatures",
    SocialMediaAccounts = "Social Media Accounts",
    SocialMediaMarketing = "Social Media Marketing",
    Spreadsheets = "Spreadsheets",
    Support = "Support",
    Taxes = "Taxes",
    TeamChat = "Team Chat",
    TeamCollaboration = "Team Collaboration",
    TimeTrackingSoftware = "Time Tracking Software",
    TaskManagement = "Task Management",
    TransactionalEmail = "Transactional Email",
    Transcription = "Transcription",
    URLShortener = "URL Shortener",
    VideoAudio = "Video & Audio",
    VideoConferencing = "Video Conferencing",
    Webinars = "Webinars",
    WebsiteAppBuilding = "Website & App Building"
}
export declare const defaultApp: App;
export {};
