export type App = {
  name: string;
  slug?: string;
  description?: string;
  tagline?: string;
  url?: string | URL;
  logo?: string | URL | Image;
  company?: Company;
  businessModel?: LeanCanvas;
  brand?: Brand;
  marketing?: Marketing;
  category?: Category | Category[];
  alternatives?: { [key: string]: Alternative };
  features?: Feature[];
  benefits?: Benefit[];
  nouns?: { [key: string]: Noun };
  functions?: { [key: string]: Function };
  triggers?: { [key: string]: Trigger };
  searches?: { [key: string]: Search };
  actions?: { [key: string]: Action };
  authentication?: { [key: string]: AuthenticationProvider };
  integrations?: Integrations;
};

export const generateApp = (app: App): App => {
  return { ...defaultApp, ...app };
};

export type AppContext = {
  app: App
  env: {}
  user: User
}

export type Persona =
  | string
  | {
      name: string;
      description?: string;
    };

export type Problem =
  | string
  | {
      problem: string | string[];
      existingAlternatives?: string | string[];
    };

export type RevenueStream = string;
export type Solution = string;
export type UniqueValueProp = string;
export type Channel = string;
export type KeyMetric = string;
export type CostStructure = string;
export type UnfairAdvantage = string;

export type LeanCanvas = {
  customerSegments?: Persona | Persona[];
  problem?: Problem | Problem[];
  revenueStreams?: RevenueStream | RevenueStream[];
  solution?: Solution | Solution[];
  uniqueValueProp?: UniqueValueProp | UniqueValueProp[];
  channels?: Channel | Channel[];
  keyMetrics?: KeyMetric | KeyMetric[];
  costStructure?: CostStructure | CostStructure[];
  unfairAdvantage?: UnfairAdvantage | UnfairAdvantage[];
};

export type Image = {
  src: string;
  height?: number;
  width?: number;
};

export type ABTester<T> = {
  value: T;
  experiment?: string;
  name?: string;
  weight?: number;
};

export type Function = <T = void>() => T;
//const handler = (req: Request, res: Response) => void

export type Company = {
  name?: string;
  legalName?: string;
  url?: string;
  address?: string;
  address2?: string;
  city?: string;
  state?: string;
  zip?: string;
  country?: string;
  phone?: string;
  email?: string;
};

export type Marketing = {};

export type Brand = {
  style: string;
  color: string;
};

export type Integrations = {
  googleAnalytics?: string;
  googleTagManager?: string;
  sentry?: string;
  rollbar?: string;
  segment?: string;
  moesif?: string;
  readme?: string;
  intercom?: string;
  mixpanel?: string;
  crisp?: string;
  stripe?: {
    publishableKey: string;
    secretKey: string;
  };
};

export type Feature =
  | string
  | {
      name: string;
      description?: string;
      icon?: string;
    };

export type Benefit =
  | string
  | {
      name: string;
      description?: string;
      icon?: string;
    };

export type User = {};

export type MultiTenantCRUD = {
  tenant: Tenant;
};

export type Tenant = {};
export type Role = {};
export type AuthenticationProvider = {};

export type Nouns = {
  [key: string]: Noun;
};

export const OneToMany = <T extends Noun<T>>(obj?: T) => obj
export const OneToOne = <T extends Noun<T>>(obj?: T) => obj
export const ManyToOne = <T extends Noun<T>>(obj?: T) => obj
export const ManyToMany = <T extends Noun<T>>(obj?: T) => obj

export type Noun<T = void> = {
  name?: string
  plural?: string
  description?: string
  schema?: Schema
  onCreate?: (instance: T, ctx: AppContext) => void | T
  onUpdate?: (instance: T, ctx: AppContext) => void | T
  onDelete?: (instance: T, ctx: AppContext) => void | T
}

export type Schema = {};

export type Trigger = {
  name?: string;
};

export type Search = {
  name?: string;
};

export type Action = {
  name?: string;
};

export type Alternative = {
  name?: string;
  url?: string;
  description?: string;
  pros?: string[];
  cons?: string[];
};

enum Category {
  Accounting = 'Accounting',
  AdsConversion = 'Ads & Conversion',
  Analytics = 'Analytics',
  AppBuilder = 'App Builder',
  AppFamilies = 'App Families',
  BookmarkManagers = 'Bookmark Managers',
  BusinessIntelligence = 'Business Intelligence',
  Calendar = 'Calendar',
  CallTracking = 'Call Tracking',
  CMS = 'Website Builders',
  Commerce = 'Commerce',
  Communication = 'Communication',
  ContactManagement = 'Contact Management',
  ContentFiles = 'Content & Files',
  CRM = 'CRM (Customer Relationship Management)',
  CustomerAppreciation = 'Customer Appreciation',
  CustomerSupport = 'Customer Support',
  Dashboards = 'Dashboards',
  Databases = 'Databases',
  DeveloperTools = 'Developer Tools',
  Devices = 'Devices',
  Documents = 'Documents',
  DripEmails = 'Drip Emails',
  eCommerce = 'eCommerce',
  Education = 'Education',
  Email = 'Email',
  EmailNewsletters = 'Email Newsletters',
  EventManagement = 'Event Management',
  Fax = 'Fax',
  FileManagementStorage = 'File Management & Storage',
  Filters = 'Filters',
  Fitness = 'Fitness',
  FormsSurveys = 'Forms & Surveys',
  Fundraising = 'Fundraising',
  Gaming = 'Gaming',
  HRTalentRecruitment = 'HR Talent & Recruitment',
  HumanResources = 'Human Resources',
  ImagesDesign = 'Images & Design',
  InternetOfThings = 'Internet of Things',
  ProposalInvoiceManagement = 'Proposal & Invoice Management',
  ITOperations = 'IT Operations',
  OnlineCourses = 'Online Courses',
  LifestyleEntertainment = 'Lifestyle & Entertainment',
  Marketing = 'Marketing',
  MarketingAutomation = 'Marketing Automation',
  NewsLifestyle = 'News & Lifestyle',
  Notes = 'Notes',
  Notifications = 'Notifications',
  PaymentProcessing = 'Payment Processing',
  PhoneSMS = 'Phone & SMS',
  Printing = 'Printing',
  ProductManagement = 'Product Management',
  Productivity = 'Productivity',
  ProjectManagement = 'Project Management',
  Reviews = 'Reviews',
  SalesCRM = 'Sales & CRM',
  SchedulingBooking = 'Scheduling & Booking',
  SecurityIdentityTools = 'Security & Identity Tools',
  ServerMonitoring = 'Server Monitoring',
  Signatures = 'Signatures',
  SocialMediaAccounts = 'Social Media Accounts',
  SocialMediaMarketing = 'Social Media Marketing',
  Spreadsheets = 'Spreadsheets',
  Support = 'Support',
  Taxes = 'Taxes',
  TeamChat = 'Team Chat',
  TeamCollaboration = 'Team Collaboration',
  TimeTrackingSoftware = 'Time Tracking Software',
  TaskManagement = 'Task Management',
  TransactionalEmail = 'Transactional Email',
  Transcription = 'Transcription',
  URLShortener = 'URL Shortener',
  VideoAudio = 'Video & Audio',
  VideoConferencing = 'Video Conferencing',
  Webinars = 'Webinars',
  WebsiteAppBuilding = 'Website & App Building',
}

export const defaultApp: App = {
  name: 'SaaS App',
  slug: 'saas-app',
  description: 'SaaS App',
  tagline: 'SaaS App',
  url: 'https://saas.dev',
  logo: 'https://saas.dev/logo.png',
  company: {
    name: 'Micro.SaaS.Studio',
    legalName: 'Micro SaaS Studio LLC',
    address: '1680 Michigan Ave #700',
    city: 'Miami Beach',
    state: 'FL',
    zip: '33139',
    country: 'US',
    phone: '(305) 850-7500',
    email: 'help@saas.dev',
    url: 'https://micro.saas.studio',
  },
  businessModel: {
    customerSegments: 'Independent Developer',
    problem: 'Building a SaaS App is complex and time consuming',
    revenueStreams: 'Monthly Recurring Revenue ($ / mo MRR)',
    solution:
      'A headless SaaS-as-a-Service Platform to enable rapid SaaS creation',
    uniqueValueProp: 'Create a monetizable SaaS offering in minutes',
    channels: ['Open Source', 'Inbound SEO'],
    keyMetrics: [
      '# of Developers',
      'Time to Hello World',
      '% of Paid Conversions',
      'Monthly Recurring Revenue',
    ],
    costStructure: [
      'Cloud Computing',
      'Customer Acquisition',
      'Customer Support',
    ],
    unfairAdvantage: '',
  },
  brand: {
    style: 'modern',
    color: 'indigo',
  },
  marketing: {},
  category: Category.DeveloperTools,
  integrations: {
    googleAnalytics: 'UA-',
  },
};
